package com.zhongshu.reward.server.core.timer;

import com.github.microservice.components.data.mongo.token.service.ResourceTokenService;
import com.zhongshu.reward.client.type.TransferStatus;
import com.zhongshu.reward.server.core.dao.WxTransferBatchDao;
import com.zhongshu.reward.server.core.domain.WxTransferBatch;
import com.zhongshu.reward.server.core.service.WxTransferService;
import lombok.Cleanup;
import lombok.Setter;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.context.ApplicationContext;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Optional;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;

/**
 * @author wjf
 * @date 2024/8/22
 */
@Slf4j
@Component
@EnableScheduling
public class TransferStatusTimer implements ApplicationRunner {

    @Autowired
    WxTransferBatchDao wxTransferBatchDao;

    @Autowired
    private ResourceTokenService resourceTokenService;

    @Autowired
    private WxTransferService wxTransferService;

    private Timer transferStatusTimer = new Timer();

    private AtomicBoolean isRunTransferStatusTimer = new AtomicBoolean(false);

    private AtomicReference<ResourceTokenService.Token> token = new AtomicReference<>();
    @Override
    public void run(ApplicationArguments args) throws Exception {
        transferStatusTimer.schedule(new TimerTask() {
            @Override
            public void run() {
                if (isRunTransferStatusTimer.get()){
                    return;
                }
                isRunTransferStatusTimer.set(true);
                try {
                    //
                    updateTransferStatus();
                }catch (Exception e){
                    e.printStackTrace();
                }finally {
                    isRunTransferStatusTimer.set(false);
                }
            }
        }, 3000, 3000);

    }


    @SneakyThrows
    public synchronized void updateTransferStatus(){
        @Cleanup ResourceTokenService.Token token = resourceTokenService.token("updateTransferStatus");
        try {

            List<WxTransferBatch> byTransferStatus = wxTransferBatchDao.findByTransferStatus(TransferStatus.PROCESSING);
            if (null == byTransferStatus || byTransferStatus.isEmpty()){
                return;
            }
            for (WxTransferBatch wxTransferBatch : byTransferStatus){
                @Cleanup TaskExecutor taskExecutor = new TaskExecutor();
                taskExecutor.setWxTransferBatch(wxTransferBatch);
                taskExecutor.start();
            }
        }catch (Exception e){
            e.printStackTrace();
        }
    }


    private class TaskExecutor{
        @Setter
        private WxTransferBatch wxTransferBatch;

        private Timer timer = new Timer();

        public void start() {
            timer.scheduleAtFixedRate(new TimerTask() {
                @Override
                public void run() {

                }
            }, 5000L, 1000L);

            log.info("提现状态更新开始");
            wxTransferService.updateTransferStatus(wxTransferBatch.getId());
        }

        public void close() {
            timer.cancel();
        }
    }

    private void findAndUpdateTask(){
        List<WxTransferBatch> byTransferStatus = wxTransferBatchDao.findByTransferStatus(TransferStatus.PROCESSING);
    }
}
